unit TrayIcon;

interface

Uses
    Windows;

type
    TTrayStr = array [0..63] of Char;
    {
type
    TTrayInfo = record
        cbSize: DWORD;                     // the size of the whole structure
        Wnd: HWND;                         // the Handle of the window (form) that owns the icon
        uID: UINT;                         // just a number; ignore
        uFlags: UINT;                      // three bits indicating validity
        uCallbackMessage: UINT;            // the message it will generate
        hIcon: HICON;                      // the Handle of the icon
        szTip: TTrayStr;                   // the text of the tool tip.
    end;
    }

type
  PNewNotifyIconData = ^TTrayInfo;
  TDUMMYUNIONNAME    = record
    case Integer of
      0: (uTimeout: UINT);
      1: (uVersion: UINT);
  end;

  TTrayInfo = record
    cbSize: DWORD;
    Wnd: HWND;
    uID: UINT;
    uFlags: UINT;
    uCallbackMessage: UINT;
    hIcon: HICON;
   //Version 5.0 is 128 chars, old ver is 64 chars
    szTip: array [0..127] of Char;
    dwState: DWORD; //Version 5.0
    dwStateMask: DWORD; //Version 5.0
    szInfo: array [0..255] of Char; //Version 5.0
    DUMMYUNIONNAME: TDUMMYUNIONNAME;
    szInfoTitle: array [0..63] of Char; //Version 5.0
    dwInfoFlags: DWORD;   //Version 5.0
  end;

    procedure InitializeTrayIcon(var TrayInfo: TTrayInfo; Wnd: HWND; CallbackMessage: Cardinal);
    procedure AdjustTrayIcon(var TrayInfo: TTrayInfo; Icon: HIcon; Tip: TTrayStr; Action: Cardinal);
    procedure ShowBalloonTipsTrayIcon(var TrayInfo: TTrayInfo; Title, Description: String; TimeOut: Cardinal);

implementation

Uses
    SysUtils,
    ShellAPI;

procedure InitializeTrayIcon(var TrayInfo: TTrayInfo; Wnd: HWND; CallbackMessage: Cardinal);
begin
    TrayInfo.cbSize := SizeOf(TrayInfo);
    TrayInfo.Wnd := Wnd;
    TrayInfo.uID := 0;
    TrayInfo.uFlags := NIF_ICON or NIF_TIP or NIF_MESSAGE;
    TrayInfo.uCallbackMessage := CallbackMessage;
end;

procedure AdjustTrayIcon(var TrayInfo: TTrayInfo; Icon: HIcon; Tip: TTrayStr; Action: Cardinal);
begin
    TrayInfo.hIcon := Icon;
    StrPCopy(TrayInfo.szTip, Tip);
    Shell_NotifyIcon(Action, @TrayInfo);
end;

procedure ShowBalloonTipsTrayIcon(var TrayInfo: TTrayInfo; Title, Description: String; TimeOut: Cardinal);
var
    PreviousFlags: Cardinal;
begin
    PreviousFlags := TrayInfo.uFlags;
    TrayInfo.cbSize := SizeOf(TrayInfo);
    TrayInfo.uFlags := NIF_INFO;
    strPLCopy(TrayInfo.szInfo, Description, SizeOf(TrayInfo.szInfo) - 1);
    TrayInfo.DUMMYUNIONNAME.uTimeout := TimeOut;
    strPLCopy(TrayInfo.szInfoTitle, Title, SizeOf(TrayInfo.szInfoTitle) - 1);
    TrayInfo.dwInfoFlags := NIIF_INFO;     //NIIF_ERROR;  //NIIF_WARNING;
    Shell_NotifyIcon(NIM_MODIFY, @TrayInfo);
    TrayInfo.uFlags := PreviousFlags;
end;

end.
