//********************************************************************************************************************************
//*                                                                                                                              *
//*     TRegistration 2.1.24.299  3delite 2006-2024                                                                             *
//*     See TRegistration ReadMe.txt for details                                                                                 *
//*                                                                                                                              *
//* Two licenses are available if you like this component:                                                                       *
//* Shareware license: 125                                                                                                      *
//*     http://www.shareit.com/product.html?productid=300091298                                                                  *
//* Commercial license: 250                                                                                                     *
//*     http://www.shareit.com/product.html?productid=300091297                                                                  *
//*                                                                                                                              *
//*     https://www.3delite.hu/Object Pascal Developer Resources/tregistration.html                                              *
//*                                                                                                                              *
//* If you have any of the aforementioned please email: 3delite@3delite.hu                                                       *
//*                                                                                                                              *
//* Good coding! :)                                                                                                              *
//* 3delite                                                                                                                      *
//********************************************************************************************************************************

unit RegistrationDefs;

interface

uses
    Classes, 
    Vcl.Controls,
    Vcl.Graphics,
    Windows, 
    Vcl.Forms,
    Messages, 
    Vcl.StdCtrls,
    Vcl.ExtCtrls,
    SysUtils, 
    Vcl.Dialogs,
    RegForm,
    CalcReg, 
    IniFiles, 
    Registry,
    Helper3delite,
    Helper3deliteWin;

const
  STR_DEF_ORDER_SITE    = 'www.shareit.com';

  STR_REG_SETTINGS      = 'Software';
  STR_REG_SETTINGS_SUBKEY = 'Registration information';
  STR_REG_NAME          = 'Name';
  STR_REG_EMAIL         = 'E-Mail';
  STR_REG_REGNUM        = 'Registration number';
  STR_REG_VERCODE       = 'Version code';

type
  TBeforeRegistrationSaveEvent = procedure(Sender: TObject) of object;
  TAfterRegistrationSaveEvent = procedure(Sender: TObject) of object;
  TBeforeRegistrationLoadEvent = procedure(Sender: TObject) of object;
  TAfterRegistrationLoadEvent = procedure(Sender: TObject) of object;
  TBeforeRegistrationCheckEvent = procedure(Sender: TObject) of object;
  TRegistrationClickOrderEvent = procedure(Sender: TObject) of object;
  TRegistrationCheckerEvent = function(Sender: TObject; RegName, RegEmail, RegNumber, RegVerCode: String; RegRandomSeed, VerRandomSeed: Extended): Boolean of object;
  TAfterRegistrationCheckEvent = procedure(Sender: TObject; var Success: Boolean) of object;
  TOnDoRegistrationClickEvent = procedure(Sender: TObject; Success, ShowDialog: Boolean) of object;
  TOnNotEnoughInfoForReqVerCodeEvent = procedure(Sender: TObject; RegName, RegEmail, RegNumber: String) of object;
  TOnSendReqVerCodeEvent = procedure(Sender: TObject; RegName, RegEmail, RegNumber: String) of object;
  TOnRegistrationDialogCreateEvent = procedure(Sender: TObject; Form: TForm) of object;

type
  TSettingsStoreMode = (ssmIniFile, ssmRegistry);
  TRegRoot = (rrUnknown, rrHKEY_LOCAL_MACHINE, rrHKEY_CURRENT_USER, rrHKEY_CLASSES_ROOT, rrHKEY_USERS, rrHKEY_CURRENT_CONFIG);

  function GetLoginName: string;
  function CheckReg(Name, Email, RegNum, VerCode: String; RegRandomSeed, VerRandomSeed: Extended): Boolean;

//* The TRegistration class
type
  TRegistration = class(TComponent)
  private
    fTextAppName: String;
    fTextAuthor: String;
    fTextPrice: String;
    fTextOrderSite: String;
    fTextProductID: String;
    fURLOrderForm: String;
    fURLRegistration: String;
    fVerCodeUpdateEmail: String;
    fIniFileName: String;
    fRegistryRoot: HKEY;
    fRegistryKeyName: String;
    fSettingsStoreMode: TSettingsStoreMode;
    fRandomSeedReg: Extended;
    fRandomSeedVer: Extended;
    fRegName: String;
    fRegEmail: String;
    fRegNumber: String;
    fRegVerCode: String;
    //* Events
    eBeforeRegistrationSave: TBeforeRegistrationSaveEvent;
    eAfterRegistrationSave: TAfterRegistrationSaveEvent;
    eBeforeRegistrationLoad: TBeforeRegistrationLoadEvent;
    eAfterRegistrationLoad: TAfterRegistrationLoadEvent;
    eBeforeRegistrationCheck: TBeforeRegistrationCheckEvent;
    eAfterRegistrationCheck: TAfterRegistrationCheckEvent;
    eOnRegistrationClickOrder: TRegistrationClickOrderEvent;
    eRegistrationChecker: TRegistrationCheckerEvent;
    eOnDoRegistrationClick: TOnDoRegistrationClickEvent;
    eOnNotEnoughInfoForReqVerCode: TOnNotEnoughInfoForReqVerCodeEvent;
    eOnSendReqVerCode: TOnSendReqVerCodeEvent;
    eOnRegistrationDialogCreate: TOnRegistrationDialogCreateEvent;
    function GetRegistryRoot: TRegRoot;
    procedure SetRegistryRoot(const Value: TRegRoot);
  public
    FormRegistration: TFormReg;
    Constructor Create(AOwner: TComponent); override;
    Destructor Destroy; override;
    function Registered: Boolean;
    function ShowModal: Integer;
    procedure Show;
    procedure ShowEvaluationOver;
    function Load: Boolean;
    function Save: Boolean;
    function LoadSettings(FileName: String): Boolean;
    function LoadSettingsFromRegistry(KeyName: String; RootKey: Cardinal): Boolean;
    function SaveSettings(FileName: String): Boolean;
    function SaveSettings2Registry(KeyName: String; RootKey: Cardinal): Boolean;
    function CheckVersionCode(RegNum, VerCode: String; VerRandomSeed: Extended): Boolean;
    function CheckRegistrationNumber(Name, Email, RegNum: String; RegRandomSeed: Extended): Boolean;
  published
    property TextAppName: String read fTextAppName write fTextAppName;
    property TextAuthor: String read fTextAuthor write fTextAuthor;
    property TextPrice: String read fTextPrice write fTextPrice;
    property TextProductID: String read fTextProductID write fTextProductID;
    property TextOrderSite: String read fTextOrderSite write fTextOrderSite;
    property URLOrderForm: String read fURLOrderForm write fURLOrderForm;
    property URLRegistration: String read fURLRegistration write fURLRegistration;
    property VerCodeUpdateEmailAddress: String read fVerCodeUpdateEmail write fVerCodeUpdateEmail;
    property SettingsStoreMode: TSettingsStoreMode read fSettingsStoreMode write fSettingsStoreMode default ssmIniFile;
    property IniFileName: String read fIniFileName write fIniFileName;
    property RegistryRoot: TRegRoot read GetRegistryRoot write SetRegistryRoot;
    property RegistryKeyName: String read fRegistryKeyName write fRegistryKeyName;
    property RandomSeedReg: Extended read fRandomSeedReg write fRandomSeedReg;
    property RandomSeedVer: Extended read fRandomSeedVer write fRandomSeedVer;
    property RegName: String read fRegName write fRegName;
    property RegEmail: String read fRegEmail write fRegEmail;
    property RegNumber: String read fRegNumber write fRegNumber;
    property RegVerCode: String read fRegVerCode write fRegVerCode;
    //* Events
    property BeforeRegistrationSave: TBeforeRegistrationSaveEvent read eBeforeRegistrationSave write eBeforeRegistrationSave;
    property AfterRegistrationSave: TAfterRegistrationSaveEvent read eAfterRegistrationSave write eAfterRegistrationSave;
    property BeforeRegistrationLoad: TBeforeRegistrationLoadEvent read eBeforeRegistrationLoad write eBeforeRegistrationLoad;
    property AfterRegistrationLoad: TAfterRegistrationLoadEvent read eAfterRegistrationLoad write eAfterRegistrationLoad;
    property BeforeRegistrationCheck: TBeforeRegistrationCheckEvent read eBeforeRegistrationCheck write eBeforeRegistrationCheck;
    property AfterRegistrationCheck: TAfterRegistrationCheckEvent read eAfterRegistrationCheck write eAfterRegistrationCheck;
    property OnRegistrationClickOrder: TRegistrationClickOrderEvent read eOnRegistrationClickOrder write eOnRegistrationClickOrder;
    property RegistrationChecker: TRegistrationCheckerEvent read eRegistrationChecker write eRegistrationChecker;
    property OnDoRegistrationClick: TOnDoRegistrationClickEvent read eOnDoRegistrationClick write eOnDoRegistrationClick;
    property OnNotEnoughInfoForReqVerCode: TOnNotEnoughInfoForReqVerCodeEvent read eOnNotEnoughInfoForReqVerCode write eOnNotEnoughInfoForReqVerCode;
    property OnSendReqVerCode: TOnSendReqVerCodeEvent read eOnSendReqVerCode write eOnSendReqVerCode;
    property OnRegistrationDialogCreate: TOnRegistrationDialogCreateEvent read eOnRegistrationDialogCreate write eOnRegistrationDialogCreate;
end;

procedure Register;

implementation

Uses
    DialogRegister;

procedure Register;
begin
    RegisterComponents('3delite', [TRegistration]);
end;

function GetLoginName: string;
var
  Buffer: Array[0..255] of Char;
  Size: dword;
begin
    Size := High(Buffer) + 1;
    if GetUserName(Buffer, Size)
        then Result := Buffer
        else Result := '';
end;

function CheckReg(Name, Email, RegNum, VerCode: String; RegRandomSeed, VerRandomSeed: Extended): Boolean;
begin
    Result := False;
    try
        if IntToStr(CalcRegNum(Name, Email, RegRandomSeed)) = Copy(RegNum, 7, 6) then begin
            if IntToStr(CalcVerCode(StrToInt64(RegNum), VerRandomSeed)) = VerCode then begin
                Result := True;
            end;
        end;
    except
        Result := False;
    end;
end;

constructor TRegistration.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    if ComponentState = [csDesigning, csFreeNotification] then begin
        fTextOrderSite := STR_DEF_ORDER_SITE;
        fRegistryRoot := HKEY_CURRENT_USER;
        Randomize;
        fRandomSeedReg := Random * 2;
        fRandomSeedVer := Random * 5;
    end;
    if NOT (csDesigning in ComponentState) then begin
        if fIniFileName = ''
            then fIniFileName := ChangeFileExt(Application.ExeName, '.ini');
        if fTextAppName = ''
            then fTextAppName := GetFileDescriptionStr(Application.ExeName);
        if fTextAuthor = ''
            then fTextAuthor := GetFileAuthorStr(Application.ExeName);
        if fRegistryRoot = 0
            then fRegistryRoot := HKEY_CURRENT_USER;
        if fRegistryKeyName = ''
            then fRegistryKeyName := STR_REG_SETTINGS + '\' + fTextAuthor + '\' + fTextAppName + '\' + STR_REG_SETTINGS_SUBKEY;
    end;
    {$IFDEF FORWINSTORE}
    //*
    {$ELSE}
    //FormRegistration := TFormReg.Create(Self);
    {$ENDIF}
end;

Destructor TRegistration.Destroy;
begin
    FreeAndNil(FormRegistration);
    inherited Destroy;
end;

function TRegistration.Registered: Boolean;
begin
    Result := False;
    try
        if Assigned(eBeforeRegistrationCheck) then begin
            eBeforeRegistrationCheck(Self);
        end;
    except
        //*
    end;
    try
        {$IFDEF FORWINSTORE}
        Result := True;
        {$ELSE}
        if NOT Assigned(FormRegistration)
            then FormRegistration := TFormReg.Create(Self);
        if Assigned(eRegistrationChecker)
            then Result := eRegistrationChecker(Self, fRegName, fRegEmail, fRegNumber, fRegVerCode, fRandomSeedReg, fRandomSeedVer)
            else Result := CheckReg(fRegName, fRegEmail, fRegNumber, fRegVerCode, fRandomSeedReg, fRandomSeedVer);
        {$ENDIF}
    except
        Result := False;
    end;
    try
        if Assigned(eAfterRegistrationCheck) then begin
            eAfterRegistrationCheck(Self, Result);
        end;
    except
        //*
    end;
end;

function TRegistration.CheckVersionCode(RegNum, VerCode: String; VerRandomSeed: Extended): Boolean;
begin
    Result := False;
    try
        if IntToStr(CalcVerCode(StrToInt64(RegNum), VerRandomSeed)) = VerCode then begin
            Result := True;
        end;
    except
        Result := False;
    end;
end;

function TRegistration.CheckRegistrationNumber(Name, Email, RegNum: String; RegRandomSeed: Extended): Boolean;
begin
    Result := False;
    try
        if IntToStr(CalcRegNum(Name, Email, RegRandomSeed)) = Copy(RegNum, 7, 6) then begin
            Result := True;
        end;
    except
        Result := False;
    end;
end;

function TRegistration.ShowModal: Integer;
begin
    {$IFDEF FORWINSTORE}
    Result := mrOk;
    Exit;
    {$ELSE}
    Result := 0;
    try
        if NOT Assigned(FormRegistration)
            then FormRegistration := TFormReg.Create(Self);
        Result := FormRegistration.ShowModal;
    except
        //*
    end;
    {$ENDIF}
end;

procedure TRegistration.Show;
begin
    {$IFDEF FORWINSTORE}
    Exit;
    {$ELSE}
    try
        if NOT Assigned(FormRegistration)
            then FormRegistration := TFormReg.Create(Self);
        FormRegistration.Show;
    except
        //*
    end;
    {$ENDIF}
end;

procedure TRegistration.ShowEvaluationOver;
begin
    {$IFDEF FORWINSTORE}
    Exit;
    {$ELSE}
    if NOT Assigned(FormRegister) then begin
        FormRegister := TFormRegister.Create(Self);
    end;
    FormRegister.ShowModal;
    {$ENDIF}
end;

function TRegistration.Load: Boolean;
begin
    Result := False;
    try
        if Assigned(eBeforeRegistrationLoad)
            then eBeforeRegistrationLoad(Self);
        case fSettingsStoreMode of
            ssmIniFile: Result := LoadSettings(fIniFileName);
            ssmRegistry: Result := LoadSettingsFromRegistry(fRegistryKeyName, fRegistryRoot);
        end;
        if Assigned(eAfterRegistrationLoad)
            then eAfterRegistrationLoad(Self);
    except
        Result := False;
    end;
end;

function TRegistration.Save: Boolean;
begin
    Result := False;
    try
        if Assigned(eBeforeRegistrationSave)
            then eBeforeRegistrationSave(Self);
        case fSettingsStoreMode of
            ssmIniFile: Result := SaveSettings(fIniFileName);
            ssmRegistry: Result := SaveSettings2Registry(fRegistryKeyName, fRegistryRoot);
        end;
        if Assigned(eAfterRegistrationSave)
            then eAfterRegistrationSave(Self);
    except
        Result := False;
    end;
end;

function TRegistration.LoadSettings(FileName: String): Boolean;
var
  IniFile: TIniFile;
begin
    //Result := False;
    IniFile := TIniFile.Create(FileName);
    try
        fRegName := IniFile.ReadString('RegistrationInformation','Name', GetLoginName);
        fRegEmail := IniFile.ReadString('RegistrationInformation','Email', '');
        fRegNumber := IniFile.ReadString('RegistrationInformation','RegistrationNumber', '');
        fRegVerCode := IniFile.ReadString('RegistrationInformation','VersionCode', '');
        Result := True;
    finally
        IniFile.Free;
    end;
end;

function TRegistration.LoadSettingsFromRegistry(KeyName: String; RootKey: Cardinal): Boolean;
var
  Reg: TRegistry;
begin
    //Result := False;
    Reg := TRegistry.Create;
    try
        if RootKey = 0
            then Reg.RootKey := HKEY_CURRENT_USER
            else Reg.RootKey := RootKey;
        Reg.OpenKey(KeyName + '\', False);
        if Reg.ValueExists(STR_REG_NAME)
            then fRegName := Reg.ReadString(STR_REG_NAME)
            else fRegName := GetLoginName;
        if Reg.ValueExists(STR_REG_EMAIL)
            then fRegEmail := Reg.ReadString(STR_REG_EMAIL)
            else fRegEmail := '';
        if Reg.ValueExists(STR_REG_REGNUM)
            then fRegNumber := Reg.ReadString(STR_REG_REGNUM)
            else fRegNumber := '';
        if Reg.ValueExists(STR_REG_VERCODE)
            then fRegVerCode := Reg.ReadString(STR_REG_VERCODE)
            else fRegVerCode := '';
        Result := True;
    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

function TRegistration.SaveSettings(FileName: String): Boolean;
var
  IniFile  : TIniFile;
begin
    Result := False;
    if NOT FileExists(FileName) then begin
        if NOT CreateUnicodeFile(FileName) then begin
            Exit;
        end;
    end;
    IniFile := TIniFile.Create(FileName);
    try
        With IniFile do begin
            WriteString('RegistrationInformation','Name', fRegName);
            WriteString('RegistrationInformation','Email', fRegEmail);
            WriteString('RegistrationInformation','RegistrationNumber', fRegNumber);
            WriteString('RegistrationInformation','VersionCode', fRegVerCode);
            //WriteBool('RegistrationInformation','Demo', NOT Registered);
        end;
        Result := True;
    finally
        IniFile.Free;
    end;
end;

function TRegistration.SaveSettings2Registry(KeyName: String; RootKey: Cardinal): Boolean;
var
  Reg: TRegistry;
begin
    //Result := False;
    Reg := TRegistry.Create;
    try
        if RootKey = 0
            then Reg.RootKey := HKEY_CURRENT_USER
            else Reg.RootKey := RootKey;
        Reg.OpenKey(KeyName + '\', True);
        Reg.WriteString(STR_REG_NAME, fRegName);
        Reg.WriteString(STR_REG_EMAIL, fRegEmail);
        Reg.WriteString(STR_REG_REGNUM, fRegNumber);
        Reg.WriteString(STR_REG_VERCODE, fRegVerCode);
        Result := True;
    finally
        Reg.CloseKey;
        Reg.Free;
    end;
end;

function TRegistration.GetRegistryRoot: TRegRoot;
begin
    if fRegistryRoot = HKEY_CLASSES_ROOT then begin
        Result := rrHKEY_CLASSES_ROOT;
    end else if fRegistryRoot = HKEY_CURRENT_USER then begin
        Result := rrHKEY_CURRENT_USER;
    end else if fRegistryRoot = HKEY_LOCAL_MACHINE then begin
        Result := rrHKEY_LOCAL_MACHINE;
    end else if fRegistryRoot = HKEY_CURRENT_CONFIG then begin
        Result := rrHKEY_CURRENT_CONFIG;
    end else if fRegistryRoot = HKEY_USERS then begin
        Result := rrHKEY_USERS;
    end else begin
        Result := rrUnknown;
    end;
end;

procedure TRegistration.SetRegistryRoot(const Value: TRegRoot);
begin
    case Value of
        rrHKEY_CLASSES_ROOT: fRegistryRoot := HKEY_CLASSES_ROOT;
        rrHKEY_CURRENT_USER: fRegistryRoot := HKEY_CURRENT_USER;
        rrHKEY_LOCAL_MACHINE: fRegistryRoot := HKEY_LOCAL_MACHINE;
        rrHKEY_CURRENT_CONFIG: fRegistryRoot := HKEY_CURRENT_CONFIG;
        rrHKEY_USERS: fRegistryRoot := HKEY_USERS;
    end;
end;

end.
